@@ -139,12 +139,15 @@ class type ['element] document = object
   method getElementById : js_string t -> 'element t opt meth
   method getElementsByTagName : js_string t -> 'element nodeList t meth
   method importNode : element t -> bool t -> 'element t meth
+  method adoptNode : element t -> 'element t meth
 end
 
 (** {2 Helper functions} *)
 
 val insertBefore : #node t -> #node t -> #node t opt -> unit
-  (** The expression [insertBefore n c p] behave the same as
+  (** [insertBefore n c p] inserts node [c] as child of node [n],
+      just before node [p], or as last child if [p] is empty.
+      The expression [insertBefore n c p] behave the same as
       [n##insertBefore(c, p)] but avoid the need of coercing the
       different objects to [node t]. *)
 val replaceChild : #node t -> #node t -> #node t -> unit
