@@ -131,6 +131,12 @@ type (-'a, -'b) event_listener
       ['a] is the type of the target object; the second parameter
       ['b] is the type of the event object. *)
 
+type mouse_button =
+  | No_button
+  | Left_button
+  | Middle_button
+  | Right_button
+
 class type event = object
   method _type : js_string t readonly_prop
   method target : element t optdef readonly_prop
@@ -146,8 +152,14 @@ and mouseEvent = object
   method clientY : int readonly_prop
   method screenX : int readonly_prop (* Relative to the edge of the screen *)
   method screenY : int readonly_prop
+  method ctrlKey : bool t readonly_prop
+  method shiftKey : bool t readonly_prop
+  method altKey : bool t readonly_prop
+  method metaKey : bool t readonly_prop
+  method which : mouse_button optdef readonly_prop
 
   (* Legacy methods *)
+  method button : int readonly_prop
   method fromElement : element t opt optdef readonly_prop
   method toElement : element t opt optdef readonly_prop
   method pageX : int optdef readonly_prop
@@ -161,11 +173,11 @@ and keyboardEvent = object
   method keyIdentifier : js_string t optdef readonly_prop
 end
 
-and mousewheelEvent = object (* All browsers but Firefox *)
+and wheelEvent = object (* All browsers but Firefox *)
   inherit mouseEvent
-  method wheelDelta : int readonly_prop
-  method wheelDeltaX : int optdef readonly_prop
-  method wheelDeltaY : int optdef readonly_prop
+  method delta : int readonly_prop
+  method deltaX : int optdef readonly_prop
+  method deltaY : int optdef readonly_prop
 end
 
 and mouseScrollEvent = object (* Firefox *)
@@ -196,6 +208,25 @@ and popStateEvent = object
   method state : Js.Unsafe.any readonly_prop
 end
 
+and storageEvent = object
+  inherit event
+  method key : js_string t readonly_prop
+  method oldValue : js_string t optdef readonly_prop
+  method keynewValue : js_string t optdef readonly_prop
+  method url : js_string t readonly_prop
+  method storageArea : storage t optdef readonly_prop
+end
+
+(** Storage *)
+and storage = object
+  method length : int readonly_prop
+  method key : int -> js_string t optdef meth
+  method getItem : js_string t -> js_string t optdef meth
+  method setItem : js_string t -> unit meth
+  method removeItem : js_string t -> unit meth
+  method clear : unit meth
+end
+
 (** {2 HTML elements} *)
 
 (** Properties common to all HTML elements *)
@@ -846,6 +877,9 @@ class type window = object
   method blur : unit meth
   method scroll : int -> int -> unit meth
 
+  method sessionStorage : storage t readonly_prop
+  method localStorage : storage t readonly_prop
+
   method top : window t readonly_prop
   method parent : window t readonly_prop
   method frameElement : element t opt readonly_prop
@@ -943,7 +977,7 @@ module Event : sig
   val keydown : keyboardEvent t typ
   val keyup : keyboardEvent t typ
 
-  val mousewheel : mousewheelEvent t typ
+  val mousewheel : wheelEvent t typ
   val _DOMMouseScroll : mouseScrollEvent t typ
 end
 
@@ -969,6 +1003,10 @@ val addMousewheelEventListener :
 
 (****)
 
+(** {2 Mouse event helper functions} *)
+
+val buttonPressed : #mouseEvent Js.t -> mouse_button
+
 (** {2 Position helper functions} *)
 
 val eventAbsolutePosition : #mouseEvent t -> int * int
@@ -1138,7 +1176,24 @@ type taggedElement =
 val tagged : #element t -> taggedElement
 val opt_tagged : #element t opt -> taggedElement option
 
+type taggedEvent =
+  | MouseEvent of mouseEvent t
+  | KeyboardEvent of keyboardEvent t
+  | WheelEvent of wheelEvent t
+  | MouseScrollEvent of mouseScrollEvent t
+  | PopStateEvent of popStateEvent t
+  | OtherEvent of event t
+
+val taggedEvent : #event t -> taggedEvent
+val opt_taggedEvent : #event t opt -> taggedEvent option
+
+val stopPropagation : #event t -> unit
+
+
 module CoerceTo : sig
+
+  (** HTMLElement *)
+
   val element : #Dom.node t -> element t opt
 
   val a : #element t -> anchorElement t opt
@@ -1199,4 +1254,17 @@ module CoerceTo : sig
   val title : #element t -> titleElement t opt
   val tr : #element t -> tableRowElement t opt
   val ul : #element t -> uListElement t opt
+
+  (** Event *)
+
+  val mouseEvent : #event t -> mouseEvent t opt
+  val keyboardEvent : #event t -> keyboardEvent t opt
+  val wheelEvent : #event t -> wheelEvent t opt
+  val mouseScrollEvent : #event t -> mouseScrollEvent t opt
+  val popStateEvent : #event t -> popStateEvent t opt
+
 end
+
+(**/**)
+
+val onIE : bool
