@@ -165,17 +165,20 @@ function caml_compare_val (a, b, total) {
     if (!(total && a === b)) {
       if (a instanceof MlString) {
         if (b instanceof MlString) {
-          if (a != b) return a.compare(b);
+            if (a != b) {
+		var x = a.compare(b);
+		if (x != 0) return x;
+	    }
         } else
           // Should not happen
           return 1;
-      } else if (a instanceof Array && a[0] == (a[0]|0)) {
+      } else if (a instanceof Array && a[0] === (a[0]|0)) {
         // Forward object
         var ta = a[0];
         if (ta === 250) {
           a = a[1];
           continue;
-        } else if (b instanceof Array && b[0] == (b[0]|0)) {
+        } else if (b instanceof Array && b[0] === (b[0]|0)) {
           // Forward object
           var tb = b[0];
           if (tb === 250) {
@@ -185,12 +188,18 @@ function caml_compare_val (a, b, total) {
             return (ta < tb)?-1:1;
           } else {
             switch (ta) {
-            case 248:
-              // Object
-              return caml_int_compare(a[2], b[2]);
-            case 255:
-              // Int64
-              return caml_int64_compare(a, b);
+            case 248: {
+		// Object
+		var x = caml_int_compare(a[2], b[2]);
+		if (x != 0) return x;
+		break;
+	    }
+            case 255: {
+		// Int64
+		var x = caml_int64_compare(a, b);
+		if (x != 0) return x;
+		break;
+	    }
             default:
               if (a.length != b.length) return (a.length < b.length)?-1:1;
               if (a.length > 1) stack.push(a, b, 1);
@@ -199,7 +208,7 @@ function caml_compare_val (a, b, total) {
         } else
           return 1;
       } else if (b instanceof MlString ||
-                 (b instanceof Array && b[0] == (b[0]|0))) {
+                 (b instanceof Array && b[0] === (b[0]|0))) {
         return -1;
       } else {
         if (a < b) return -1;
@@ -446,7 +455,7 @@ function caml_hash_univ_param (count, limit, obj) {
   function hash_aux (obj) {
     limit --;
     if (count < 0 || limit < 0) return;
-    if (obj instanceof Array && obj[0] == (obj[0]|0)) {
+    if (obj instanceof Array && obj[0] === (obj[0]|0)) {
       switch (obj[0]) {
       case 248:
         // Object
@@ -476,11 +485,11 @@ function caml_hash_univ_param (count, limit, obj) {
         for (var i = 0; i < l; i++)
           hash_accu = (hash_accu * 19 + b.charCodeAt(i)) | 0;
       }
-    } else if (obj == (obj|0)) {
+    } else if (obj === (obj|0)) {
       // Integer
       count --;
       hash_accu = (hash_accu * 65599 + obj) | 0;
-    } else if (obj == +obj) {
+    } else if (obj === +obj) {
       // Float
       count--;
       var p = caml_int64_to_bytes (caml_int64_bits_of_float (obj));
